#ifndef QCDNUM_H
#define QCDNUM_H

#include <string>
#include <iostream>
using namespace std;

namespace MBUTIL {

    /************************************************************/
    /*  MBUTIL hash routines from hash.f                        */
    /************************************************************/

    int imb_ihash(int iseed, int *imsg, int n);
    int imb_jhash(int iseed, double *dmsg, int n);
    int imb_dhash(int iseed, double *dmsg, int n);

    /************************************************************/
    /*  MBUTIL character routines from mchar.f                  */
    /************************************************************/

    void smb_hcode(int ihash, string &hcode);

    /************************************************************/
    /*  MBUTIL workspace routines from wspace.f                 */
    /************************************************************/

    inline int iaFtoC(int ia) { return ia-1; };
    inline int iaCtoF(int ia) { return ia+1; };

    int imb_wsinit(double *w, int nw, int nt );
    void smb_setwsn(double *w, int nw);
    int imb_newset(double *w);
    int imb_wtable(double* w, int* imi, int* ima, int n);
    int imb_wclone(double *obj, double *w2);
    void smb_tbcopy(double *tab1, double *tab2, int itag);
    void smb_tsdump(string fname, int key, double *tset, int &ierr);
    int imb_tsread(string fname, int key, double *w, int &ierr);
    int imb_marker(string obj);
    int imb_hdsize();
    int imb_tbsize(int *imi, int *ima, int ndim);
    int imb_tpoint(double *table, int* index, int n);

    int imb_IsaWorkspace(double *w);
    int imb_SizeOfW(double *obj);
    int imb_WordsUsed(double *obj);
    int imb_Nheader(double *obj);
    int imb_Ntags(double *obj);
    int imb_HeadSkip(double *obj);
    int imb_IaDrain(double *obj);
    int imb_IaNull(double *obj);
    int imb_ObjectType(double *obj);
    int imb_ObjectSize(double *obj);
    int imb_Nobjects(double *obj);
    int imb_ObjectNumber(double *obj);
    int imb_FingerPrint(double *obj);
    int imb_IaFirstTag(double *obj);
    int imb_TableDim(double *obj);
    int imb_IaKARRAY(double *obj);
    int imb_IaIMIN(double *obj);
    int imb_IaIMAX(double *obj);
    int imb_BeginTbody(double *obj);
    int imb_EndTbody(double *obj);
    int imb_dchdir(double *obj);
    int imb_dfskip(double *obj);
    int imb_dbskip(double *obj);
    void smb_wstree(double *w);
    void smb_wshead(double *obj);
}

namespace QCDNUM {


    /************************************************************/
    /*  TOOLBOX init routines from tbx00ini.f                   */
    /************************************************************/

    void tbx_wsinit(double *w, int nw);

    /************************************************************/
    /*  TOOLBOX pointer routines from tbxpointers.f             */
    /************************************************************/


    /************************************************************/
    /*  TOOLBOX workspace routines from tbxwspace.f             */
    /************************************************************/


    /************************************************************/
    /*  QCDNUM evolution routines from usrevol.f                */
    /************************************************************/

    double rfromf(double fscale2);
    double ffromr(double rscale2);
    double asfunc(double r2, int &nf, int &ierr);
    double altabn(int iset, int iq, int n, int &ierr);
    void evolfg(int iset, double (*func)(int*, double*), double *def, int iq0, double &epsi);
    void evsgns(int iset, double (*func)(int*, double*), int *isns, int n, int iq0, double &epsi);
    void pdfcpy(int iset1, int iset2);
    void extpdf(double (*func)(int*, double*, double*, bool*), int iset, int n, double offset, double &epsi);
    void usrpdf(double (*func)(int*, double*, double*, bool*), int iset, int n, double offset, double &epsi);
    int nptabs(int iset);
    int ievtyp(int iset);

    /************************************************************/
    /*  QCDNUM grid routines from usrgrd.f                      */
    /************************************************************/

    void gxmake(double *xmin, int *iwt, int n, int nxin, int &nxout, int iord);
    int ixfrmx(double x);
    int xxatix(double x, int ix);
    double xfrmix(int ix);
    void gxcopy(double *array, int n, int &nx);
    void gqmake(double *qarr, double *wgt, int n, int nqin, int &nqout);
    int iqfrmq(double q2);
    int qqatiq(double q2, int iq);
    double qfrmiq(int iq);
    void gqcopy(double *array, int n, int &nq);
    void grpars(int &nx, double &xmi, double &xma, int &nq, double &qmi, double &qma, int &iord);
    void setlim( int ixmin, int iqmin, int iqmax, double dummy);
    void getlim( int iset, double &xmin, double &qmin, double &qmax, double &dummy);

    /************************************************************/
    /*  QCDNUM init routines from usrini.f                      */
    /************************************************************/

    void qcinit(int lun, string fname);
    void setlun(int lun, string fname);
    int nxtlun(int lmin);
    void setval(string opt, double val);
    void getval(string opt, double &val);
    void setint(string opt, int ival);
    void getint(string opt, int &ival);
    void qstore(string opt, int ival, double &val);

    /************************************************************/
    /*  QCDNUM parameter routines from usrparams.f              */
    /************************************************************/

    void setord(int iord);
    void getord(int &iord);
    void setalf(double as0, double r20);
    void getalf(double &as0, double &r20);
    void setcbt(int nfix, int iqc, int iqb, int iqt);
    void mixfns(int nfix, double r2c, double r2b, double r2t);
    void getcbt(int &nfix, double &q2c, double &q2b, double &q2t);
    void setabr(double ar, double br);
    void getabr(double &ar, double &br);
    int nfrmiq(int iset, int iq, int &ithresh);
    int nflavs(int iq, int &ithresh);
    void cpypar(double *array, int n, int iset);
    void usepar(int iset);
    int keypar(int iset);
    int keygrp(int iset, int igrp);
    void pushcp();
    void pullcp();

    /************************************************************/
    /*  QCDNUM pdf output routines from usrpdf.f                */
    /************************************************************/

    void allfxq(int iset, double x, double qmu2, double *pdfs, int n, int ichk);
    void allfij(int iset, int ix, int iq, double *pdfs, int n, int ichk);
    double bvalxq(int iset, int id, double x, double qmu2, int ichk);
    double bvalij(int iset, int id, int ix, int iq, int ichk);
    double fvalxq(int iset, int id, double x, double qmu2, int ichk);
    double fvalij(int iset, int id, int ix, int iq, int ichk);
    double sumfxq(int iset, double *c, int isel, double x, double qmu2, int ichk);
    double sumfij(int iset, double *c, int isel, int ix, int iq, int ichk);
    double fsplne(int iset, int id, double x, int iq);
    double splchk(int iset, int id, int iq);
    void fflist(int iset, double *c, int m, double *x, double *q, double *f, int n, int ichk);
    void ftable(int iset, double *c, int m, double *x, int nx, double *q, int nq, double *table, int ichk);
    void ffplot(string fnam, double(* fun)(int*, double*, bool*), int m, double zmi, double zma, int n, string txt);

    /************************************************************/
    /*  QCDNUM weight routines from usrwgt.f                    */
    /************************************************************/

    void fillwt(int itype, int &idmin, int &idmax, int &nwds);
    void dmpwgt(int itype, int lun, string fname);
    void readwt(int lun, string fname, int &idmin, int &idmax, int &nwds, int &ierr);
    void wtfile(int itype, string fname);
    void nwused(int &nwtot, int &nwuse, int &ndummy);

    /************************************************************/
    /*  HQSTF routines                                          */
    /************************************************************/

    void hqwords(int &ntotal, int &nused);
    void hqparms(double *qmass, double &a, double &b);
    double hqqfrmu(double qmu2);
    double hqmufrq(double Q2);
    void hswitch(int iset);
    void hqstfun(int istf, int icbt, double *def, double *x, double *Q2, double *f, int n, int ichk);
    void hqfillw(int istf, double *qmass, double aq, double bq, int &nused);
    void hqdumpw(int lun, string fname);
    void hqreadw(int lun, string fname, int &nused, int &ierr);

    /************************************************************/
    /*  ZMSTF routines                                          */
    /************************************************************/

    void zmwords(int &ntotal, int &nused);
    void zmdefq2(double a, double b);
    void zmabval(double &a, double &b);
    double zmqfrmu(double qmu2);
    double zmufrmq(double Q2);
    void zswitch(int iset);
    void zmstfun(int istf, double *def, double *x, double *Q2, double *f, int n, int ichk);
    void zmfillw(int &nused);
    void zmdumpw(int lun, string fname);
    void zmreadw(int lun, string fname, int &nused, int &ierr);
}

#endif
